@@ -2,7 +2,7 @@ package Simo::Wrapper;
 use Simo;
 use Carp;
 
-our $VERSION = '0.01_04';
+our $VERSION = '0.01_05';
 
 use Simo::Constrain qw( is_class_name is_object );
 
@@ -123,13 +123,14 @@ sub _SIMO_parse_run_methods_args{
     return $method_infos;
 }
 
-sub encode_attrs{
-    my ( $self, $encoding, @attrs ) = @_;
+sub cycle_attrs{
+    my ( $self, $code, @attrs ) = @_;
     
     my $obj = $self->obj;
-    croak "'encode_attrs' must be called from object." unless is_object( $obj );
+    croak "'cycle_attrs' must be called from object." unless is_object( $obj );
+    
+    croak "First argument must be code reference." unless ref $code eq 'CODE';
     
-    require Encode;
     foreach my $attr ( @attrs ){
         croak "'$attr' is not exist." unless $obj->can( $attr );
         
@@ -137,37 +138,52 @@ sub encode_attrs{
         
         if( ref $obj->{ $attr } eq 'ARRAY' ){
             foreach my $i ( 0 .. @{ $obj->{ $attr } } - 1 ){
-                if( ref $obj->{ $attr }[$i] ){
-                    carp "\$self->{ '$attr' }[ $i ] must be string. Encode is not done.";
-                }
-                else{
-                    $obj->{ $attr }[$i]
-                        = Encode::encode( $encoding, $obj->{ $attr }[$i] );
-                }
+                my $info = { type => 'ARRAY', attr => $attr, index => $i, self => $obj };
+                
+                $obj->{ $attr }[ $i ] = $code->( $obj->{ $attr }[ $i ], $info );
             }
         }
         elsif( ref $obj->{ $attr } eq 'HASH' ){
             foreach my $key ( keys %{ $obj->{ $attr } } ){
-                if( ref $obj->{ $attr }{ $key } ){
-                    carp "\$self->{ '$attr' }{ '$key' } must be string. Encode is not done.";
-                }
-                else{
-                    $obj->{ $attr }{ $key } 
-                        = Encode::encode( $encoding, $obj->{ $attr }{ $key } );
-                }
+                my $info = { type => 'HASH', attr => $attr, key => $key, self => $obj };
+                
+                $obj->{ $attr }{ $key } = $code->( $obj->{ $attr }{ $key }, $info );
             }
         }
         else{
-            if( ref $obj->{ $attr } ){
-                carp "\$self->{ '$attr' } must be string or array ref or hash ref. Encode is not done.";
-            }
-            else{
-                $obj->{ $attr } = Encode::encode( $encoding, $obj->{ $attr } );
-            }
+            my $info = { type => 'SCALAR', attr => $attr, self => $obj };
+            $obj->{ $attr } = $code->( $obj->{ $attr }, $info );
         }
     }
 }
 
+sub encode_attrs{
+    my ( $self, $encoding, @attrs ) = @_;
+    
+    my $obj = $self->obj;
+    croak "'encode_attrs' must be called from object." unless is_object( $obj );
+    
+    require Encode;
+    $self->cycle_attrs(
+        sub{
+            my ( $val, $info ) = @_;
+            
+            my ( $type, $attr ) = @{ $info }{ qw( type attr ) };
+            
+            if( ref $val ){
+                my $warn = $type eq 'ARRAY'  ? "\$self->{ '$attr' }[ $info->{ index } ] must be string. Encode is not done." :
+                           $type eq 'HASH'   ? "\$self->{ '$attr' }{ '$info->{ key }' } must be string. Encode is not done." :
+                           $type eq 'SCALAR' ? "\$self->{ '$attr' } must be string or array ref or hash ref. Encode is not done." :
+                           '';
+                carp $warn;
+                return $val;
+            }
+            return Encode::encode( $encoding, $val );
+        },
+        @attrs
+    );
+}
+
 sub decode_attrs{
     my ( $self, $encoding, @attrs ) = @_;
     
@@ -175,42 +191,24 @@ sub decode_attrs{
     croak "'decode_attrs' must be called from object." unless is_object( $obj );
     
     require Encode;
-    foreach my $attr ( @attrs ){
-        croak "'$attr' is not exist." unless $obj->can( $attr );
-        
-        $obj->$attr unless exists $obj->{ $attr }; # initialized if attr is not called yet.
-        
-        if( ref $obj->{ $attr } eq 'ARRAY' ){
-            foreach my $i ( 0 .. @{ $obj->{ $attr } } - 1 ){
-                if( ref $obj->{ $attr }[$i] ){
-                    carp "\$self->{ '$attr' }[ $i ] must be string. Decode is not done.";
-                }
-                else{
-                    $obj->{ $attr }[$i]
-                        = Encode::decode( $encoding, $obj->{ $attr }[$i] );
-                }
+    $self->cycle_attrs(
+        sub{
+            my ( $val, $info ) = @_;
+            
+            my ( $type, $attr ) = @{ $info }{ qw( type attr ) };
+            
+            if( ref $val ){
+                my $warn = $type eq 'ARRAY'  ? "\$self->{ '$attr' }[ $info->{ index } ] must be string. Decode is not done." :
+                           $type eq 'HASH'   ? "\$self->{ '$attr' }{ '$info->{ key }' } must be string. Decode is not done." :
+                           $type eq 'SCALAR' ? "\$self->{ '$attr' } must be string or array ref or hash ref. Decode is not done." :
+                           '';
+                carp $warn;
+                return $val;
             }
-        }
-        elsif( ref $obj->{ $attr } eq 'HASH' ){
-            foreach my $key ( keys %{ $obj->{ $attr } } ){
-                if( ref $obj->{ $attr }{ $key } ){
-                    carp "\$self->{ '$attr' }{ '$key' } must be string. Decode is not done.";
-                }
-                else{
-                    $obj->{ $attr }{ $key } 
-                        = Encode::decode( $encoding, $obj->{ $attr }{ $key } );
-                }
-            }
-        }
-        else{
-            if( ref $obj->{ $attr } ){
-                carp "\$self->{ '$attr' } must be string or array ref or hash ref. Decode is not done.";
-            }
-            else{
-                $obj->{ $attr } = Encode::decode( $encoding, $obj->{ $attr } );
-            }
-        }
-    }
+            return Encode::decode( $encoding, $val );
+        },
+        @attrs
+    );
 }
 
 sub clone{
@@ -240,45 +238,6 @@ sub thaw{
     return Storable::thaw( $freezed );
 }
 
-
-sub cycle_attrs{
-    my ( $self, $code, @attrs ) = @_;
-    
-    my $obj = $self->obj;
-    croak "'cycle_attrs' must be called from object." unless is_object( $obj );
-    
-    croak "First argument must be code reference." unless ref $code eq 'CODE';
-    
-    foreach my $attr ( @attrs ){
-        croak "'$attr' is not exist." unless $obj->can( $attr );
-        
-        $obj->$attr unless exists $obj->{ $attr }; # initialized if attr is not called yet.
-        
-        if( ref $obj->{ $attr } eq 'ARRAY' ){
-            foreach my $i ( 0 .. @{ $obj->{ $attr } } - 1 ){
-                my $info = { type => 'ARRAY', attr => $attr, index => $i, self => $obj };
-                
-                my $ret = $code->( $obj->{ $attr }[ $i ], $info );
-                $obj->{ $attr }[ $i ] = $ret unless $info->{ no_ret };
-            }
-        }
-        elsif( ref $obj->{ $attr } eq 'HASH' ){
-            foreach my $key ( keys %{ $obj->{ $attr } } ){
-                my $info = { type => 'HASH', attr => $attr, key => $key, self => $obj };
-                
-                my $ret = $code->( $obj->{ $attr }{ $key }, $info );
-                $obj->{ $attr }{ $key } = $ret unless $info->{ no_ret };
-            }
-        }
-        else{
-            my $info = { type => 'SCALAR', attr => $attr, self => $obj };
-            my $ret = $code->( $obj->{ $attr }, $info );
-            $obj->{ $attr } =  $ret unless $info->{ no_ret };
-        }
-    }
-}
-
-
 =head1 NAME
 
 Simo::Wrapper - Object wrapper to manipulate attrs and methods.
@@ -287,7 +246,7 @@ Simo::Wrapper - Object wrapper to manipulate attrs and methods.
 
 =head1 VERSION
 
-Version 0.01_04
+Version 0.01_05
 
 =cut
 
